@@ -13,15 +13,20 @@
 // limitations under the License.
 
 use std::borrow::Cow;
+use std::collections::HashMap;
 use std::pin::Pin;
 use std::str::FromStr;
 use std::sync::Arc;
 use std::task::Context;
 use std::task::Poll;
 
+use anyhow::anyhow;
 use async_trait::async_trait;
 use aws_sdk_s3 as AwsS3;
-use aws_sdk_s3::error::{GetObjectError, GetObjectErrorKind, HeadObjectError, HeadObjectErrorKind};
+use aws_sdk_s3::error::GetObjectError;
+use aws_sdk_s3::error::GetObjectErrorKind;
+use aws_sdk_s3::error::HeadObjectError;
+use aws_sdk_s3::error::HeadObjectErrorKind;
 use aws_smithy_http::body::SdkBody;
 use aws_smithy_http::byte_stream::ByteStream;
 use aws_smithy_http::result::SdkError;
@@ -29,7 +34,9 @@ use futures::TryStreamExt;
 
 use crate::credential::Credential;
 use crate::error::Error;
+use crate::error::Kind;
 use crate::error::Result;
+use crate::object::Metadata;
 use crate::ops::HeaderRange;
 use crate::ops::OpDelete;
 use crate::ops::OpRead;
@@ -37,8 +44,7 @@ use crate::ops::OpStat;
 use crate::ops::OpWrite;
 use crate::readers::ReaderStream;
 use crate::Accessor;
-use crate::Object;
-use crate::Reader;
+use crate::BoxedAsyncRead;
 
 /// # TODO
 ///
@@ -106,9 +112,10 @@ impl Builder {
 
     pub async fn finish(&mut self) -> Result<Arc<dyn Accessor>> {
         if self.bucket.is_empty() {
-            return Err(Error::BackendConfigurationInvalid {
-                key: "bucket".to_string(),
-                value: "".to_string(),
+            return Err(Error::Backend {
+                kind: Kind::BackendConfigurationInvalid,
+                context: HashMap::from([("bucket".to_string(), "".to_string())]),
+                source: anyhow!("bucket is empty"),
             });
         }
 
@@ -150,9 +157,10 @@ impl Builder {
                     ));
                 }
                 _ => {
-                    return Err(Error::BackendConfigurationInvalid {
-                        key: "credential".to_string(),
-                        value: "".to_string(),
+                    return Err(Error::Backend {
+                        kind: Kind::BackendConfigurationInvalid,
+                        context: HashMap::from([("credential".to_string(), "*".to_string())]),
+                        source: anyhow!("credential is invalid"),
                     });
                 }
             }
@@ -163,19 +171,20 @@ impl Builder {
         // Load users input first, if user not input, we will fallback to aws
         // default load logic.
         if let Some(endpoint) = &self.endpoint {
-            let mut uri =
-                http::Uri::from_str(endpoint).map_err(|_| Error::BackendConfigurationInvalid {
-                    key: "endpoint".to_string(),
-                    value: endpoint.clone(),
-                })?;
+            let mut uri = http::Uri::from_str(endpoint).map_err(|e| Error::Backend {
+                kind: Kind::BackendConfigurationInvalid,
+                context: HashMap::from([("endpoint".to_string(), endpoint.clone())]),
+                source: anyhow::Error::from(e),
+            })?;
 
             let mut parts = uri.into_parts();
 
             // If uri's authority is empty, it's must be an invalid url.
             if parts.authority.is_none() {
-                return Err(Error::BackendConfigurationInvalid {
-                    key: "endpoint".to_string(),
-                    value: endpoint.clone(),
+                return Err(Error::Backend {
+                    kind: Kind::BackendConfigurationInvalid,
+                    context: HashMap::from([("endpoint".to_string(), endpoint.clone())]),
+                    source: anyhow!("uri is invalid"),
                 });
             }
 
@@ -189,9 +198,10 @@ impl Builder {
                 parts.path_and_query = Some(http::uri::PathAndQuery::from_static("/"));
             }
 
-            uri = http::Uri::from_parts(parts).map_err(|_| Error::BackendConfigurationInvalid {
-                key: "endpoint".to_string(),
-                value: endpoint.clone(),
+            uri = http::Uri::from_parts(parts).map_err(|e| Error::Backend {
+                kind: Kind::BackendConfigurationInvalid,
+                context: HashMap::from([("endpoint".to_string(), endpoint.clone())]),
+                source: anyhow::Error::from(e),
             })?;
 
             cfg = cfg.endpoint_resolver(AwsS3::Endpoint::immutable(uri));
@@ -206,6 +216,7 @@ impl Builder {
     }
 }
 
+#[derive(Debug, Clone)]
 pub struct Backend {
     bucket: String,
 
@@ -235,7 +246,7 @@ impl Backend {
 
 #[async_trait]
 impl Accessor for Backend {
-    async fn read(&self, args: &OpRead) -> Result<Reader> {
+    async fn read(&self, args: &OpRead) -> Result<BoxedAsyncRead> {
         let p = self.get_abs_path(&args.path);
 
         let mut req = self
@@ -251,12 +262,12 @@ impl Accessor for Backend {
         let resp = req
             .send()
             .await
-            .map_err(|e| parse_get_object_error(e, &args.path))?;
+            .map_err(|e| parse_get_object_error(e, "read", &args.path))?;
 
         Ok(Box::new(S3Stream(resp.body).into_async_read()))
     }
 
-    async fn write(&self, r: Reader, args: &OpWrite) -> Result<usize> {
+    async fn write(&self, r: BoxedAsyncRead, args: &OpWrite) -> Result<usize> {
         let p = self.get_abs_path(&args.path);
 
         let _ = self
@@ -270,12 +281,12 @@ impl Accessor for Backend {
             )))
             .send()
             .await
-            .map_err(|e| parse_unexpect_error(e, &args.path))?;
+            .map_err(|e| parse_unexpect_error(e, "write", &args.path))?;
 
         Ok(args.size as usize)
     }
 
-    async fn stat(&self, args: &OpStat) -> Result<Object> {
+    async fn stat(&self, args: &OpStat) -> Result<Metadata> {
         let p = self.get_abs_path(&args.path);
 
         let meta = self
@@ -285,13 +296,12 @@ impl Accessor for Backend {
             .key(&p)
             .send()
             .await
-            .map_err(|e| parse_head_object_error(e, &args.path))?;
-        let o = Object {
-            path: args.path.to_string(),
-            size: meta.content_length as u64,
-        };
+            .map_err(|e| parse_head_object_error(e, "stat", &args.path))?;
 
-        Ok(o)
+        let mut m = Metadata::default();
+        m.set_content_length(meta.content_length as u64);
+
+        Ok(m)
     }
 
     async fn delete(&self, args: &OpDelete) -> Result<()> {
@@ -304,7 +314,7 @@ impl Accessor for Backend {
             .key(&p)
             .send()
             .await
-            .map_err(|e| parse_unexpect_error(e, &args.path));
+            .map_err(|e| parse_unexpect_error(e, "delete", &args.path));
 
         Ok(())
     }
@@ -335,29 +345,68 @@ impl futures::Stream for S3Stream {
     }
 }
 
-fn parse_get_object_error(err: SdkError<GetObjectError>, path: &str) -> Error {
+fn parse_get_object_error(err: SdkError<GetObjectError>, op: &'static str, path: &str) -> Error {
     if let SdkError::ServiceError { err, .. } = err {
         match err.kind {
-            GetObjectErrorKind::NoSuchKey(_) => Error::ObjectNotExist(path.to_string()),
-            _ => Error::Unexpected(path.to_string()),
+            GetObjectErrorKind::NoSuchKey(_) => Error::Object {
+                kind: Kind::ObjectNotExist,
+                op,
+                path: path.to_string(),
+                source: anyhow::Error::from(err),
+            },
+            _ => Error::Object {
+                kind: Kind::Unexpected,
+                op,
+                path: path.to_string(),
+                source: anyhow::Error::from(err),
+            },
         }
     } else {
-        Error::Unexpected(path.to_string())
+        Error::Object {
+            kind: Kind::Unexpected,
+            op,
+            path: path.to_string(),
+            source: anyhow::Error::from(err),
+        }
     }
 }
 
-fn parse_head_object_error(err: SdkError<HeadObjectError>, path: &str) -> Error {
+fn parse_head_object_error(err: SdkError<HeadObjectError>, op: &'static str, path: &str) -> Error {
     if let SdkError::ServiceError { err, .. } = err {
         match err.kind {
-            HeadObjectErrorKind::NotFound(_) => Error::ObjectNotExist(path.to_string()),
-            _ => Error::Unexpected(path.to_string()),
+            HeadObjectErrorKind::NotFound(_) => Error::Object {
+                kind: Kind::ObjectNotExist,
+                op,
+                path: path.to_string(),
+                source: anyhow::Error::from(err),
+            },
+            _ => Error::Object {
+                kind: Kind::Unexpected,
+                op,
+                path: path.to_string(),
+                source: anyhow::Error::from(err),
+            },
         }
     } else {
-        Error::Unexpected(path.to_string())
+        Error::Object {
+            kind: Kind::Unexpected,
+            op,
+            path: path.to_string(),
+            source: anyhow::Error::from(err),
+        }
     }
 }
 
 // parse_unexpect_error is used to parse SdkError into unexpected.
-fn parse_unexpect_error<E>(_: SdkError<E>, path: &str) -> Error {
-    Error::Unexpected(path.to_string())
+fn parse_unexpect_error<E: 'static + Send + Sync + std::error::Error>(
+    err: SdkError<E>,
+    op: &'static str,
+    path: &str,
+) -> Error {
+    Error::Object {
+        kind: Kind::Unexpected,
+        op,
+        path: path.to_string(),
+        source: anyhow::Error::from(err),
+    }
 }
